import os
import subprocess
from argparse import ArgumentParser


class CLICommand:
    """命令行提权,默认为cmd
    """

    @staticmethod
    def add_arguments(parser: ArgumentParser):
        add = parser.add_argument
        add("--ps", action='store_true', help="使用powershell")

    @staticmethod
    def run(args, parser):
        current_directory = os.getcwd()
        if args.ps:
            start_ps_admin = f"Start-Process -FilePath 'powershell' -ArgumentList '-NoExit', '-Command cd \"{current_directory}\"' -Verb RunAs"
            subprocess.Popen(['powershell', '-Command', start_ps_admin])
        else:
            start_cmd_admin = f"Start-Process -FilePath 'cmd.exe' -ArgumentList '/K cd {current_directory}' -Verb RunAs"
            subprocess.Popen(['powershell', '-Command', start_cmd_admin])



import os
import subprocess
from pathlib import Path

import requests

from baosight.tools.utils import insert_tk_text


def download_gitlab_runner(runner_url, text_area):
    download_path = Path.home() / "Documents" / "gitlab-runner.exe"
    if not download_path.exists():
        insert_tk_text(text_area, "Downloading GitLab Runner...\n")
        response = requests.get(runner_url, stream=True)
        response.raise_for_status()
        with open(download_path, "wb") as f:
            for chunk in response.iter_content(chunk_size=8192):
                f.write(chunk)
        insert_tk_text(text_area, "Download completed.\n")
    else:
        insert_tk_text(text_area, "GitLab Runner already downloaded.\n")
    os.environ["PATH"] += os.pathsep + str(download_path.parent)


def unregister_existing_runners(gl, description, text_area):
    runners = gl.runners.all()
    for runner in runners:
        if runner.description == description:
            insert_tk_text(text_area, f"Unregistering existing runner: {runner.id} - {runner.description}\n")
            runner.delete()


def register_runner(gl, runner_info, registration_token, text_area):
    description = runner_info["description"]
    tags = runner_info["tags"]
    try:
        unregister_existing_runners(gl, description, text_area)
    except Exception as e:
        insert_tk_text(text_area, e.__str__())
    info = {
        'description': description,
        'active': True,
        'tag_list': tags,
        'run_untagged': True,
        'locked': False,
        'access_level': 'not_protected',
        'token': registration_token
    }
    runner = gl.runners.create(info)
    insert_tk_text(text_area, f"Runner ID: {runner.id}, Description: {description} registered.\n")
    return runner.token


def setup_runner_on_windows(token, runner_info, gitlab_url, text_area):
    service_name = runner_info['service']
    working_directory = runner_info["work_dir"]

    # 将列表转换为字符串命令的选项
    install_options = " ".join(runner_info.get("install_options", []))
    start_options = " ".join(runner_info.get("start_options", []))
    register_options = " ".join(runner_info.get("register_options", [
        "--shell powershell",
        "--executor shell",
        "--locked false",
        "--run-untagged true",
        "--access-level not_protected"
    ]))

    # 构建命令列表
    commands = [
        f"gitlab-runner stop --service {service_name}",
        f"gitlab-runner uninstall --service {service_name}",
        f"gitlab-runner install --service {service_name} --working-directory {working_directory} {install_options}",
        f"gitlab-runner start --service {service_name} {start_options}",
        f"gitlab-runner register --non-interactive --url {gitlab_url} --token {token} {register_options}"
    ]

    # 确保工作目录存在
    work_dir_path = Path(working_directory)
    work_dir_path.mkdir(parents=True, exist_ok=True)

    # 删除现有的配置文件，如果存在
    config_path = work_dir_path / "config.json"
    config_path.unlink(missing_ok=True)

    # 执行命令
    for cmd in commands:
        result = subprocess.run(cmd, cwd=work_dir_path, shell=True, text=True, capture_output=True)
        stdout = result.stdout
        insert_tk_text(text_area, f"{stdout}\n")
        if result.returncode != 0:
            insert_tk_text(text_area, f"Error: {cmd}\n")
            # break  # 如果任何命令失败，终止循环
        else:
            insert_tk_text(text_area, f"Succeed: {cmd}\n")
            print(f"Succeed: {cmd}")



import threading
from tkinter import filedialog, ttk, scrolledtext

import tkinter as tk


def center_window(root):
    """ 将窗口居中放置在屏幕上 """
    root.update_idletasks()  # 更新窗口状态
    width = root.winfo_width()
    height = root.winfo_height()
    x = (root.winfo_screenwidth() // 2) - (width // 2)
    y = (root.winfo_screenheight() // 2) - (height // 2)
    root.geometry('{}x{}+{}+{}'.format(width, height, x, y))


def create_installation_ui(deploy):
    root = tk.Tk()
    root.title("Gitlab Runner Installer")

    # 添加文件选择按钮
    config_button = tk.Button(
        root,
        text="选择配置文件",
        command=lambda: filedialog.askopenfilename(filetypes=[("JSON files", "*.json")])
    )

    progress_bar = ttk.Progressbar(root, orient='horizontal', length=500, mode='determinate', maximum=100)
    progress_bar.pack(pady=20)
    progress_bar.pack(padx=10)
    text_area = tk.Text(root, height=20, bg='black', fg='white', insertbackground='white')
    text_area.pack(fill='both', expand=True)
    start_button = tk.Button(
        root,
        text="开始安装",
        command=lambda: threading.Thread(
            target=deploy,
            kwargs={
                "gui_objs": {
                    "config_button": config_button,
                    "progress_bar": progress_bar,
                    "root": root,
                    "start_button": start_button,
                    "text_area": text_area
                }

            }
        ).start()
    )
    start_button.pack(pady=20)
    center_window(root)
    root.mainloop()



import argparse
import importlib
import textwrap

subcommands = [
    ("gl-runner", "baosight.cli.gl_runner"),
    ("cmd", "baosight.cli.cmd_admin"),
]


def main():
    parser = argparse.ArgumentParser(prog="baosight", description="常用工具包")
    subparsers = parser.add_subparsers(title="subcommand", dest="subcommand")
    subparser_help = subparsers.add_parser("help", description="Help", help="子命令帮助")
    subparser_help.add_argument("helpcommand", nargs="?", metavar="subcommand", help="为子命令提供帮助")
    parsers = {}
    functions = {}
    for subcommand, module_name in subcommands:
        CLICommand = importlib.import_module(module_name).CLICommand
        docstring = CLICommand.__doc__
        short, long = docstring.split("\n", 1)
        long = textwrap.dedent(long)
        subparser = subparsers.add_parser(subcommand, description=long, help=short)
        CLICommand.add_arguments(subparser)
        parsers[subcommand] = subparser
        functions[subcommand] = CLICommand.run
    args = parser.parse_args()  # 默认会从命令行读取参数
    if args.subcommand == "help":
        if args.helpcommand is None:
            parser.print_help()
        else:
            parsers[args.subcommand].print_help()
    elif args.subcommand is None:
        parser.print_usage()
    else:
        f = functions[args.subcommand]
        f(args, parsers[args.subcommand])


if __name__ == '__main__':
    main()



import json
from typing import Union


def validate_json(data):
    # 检查顶层键
    keys = ['gitlab_url', 'gitlab_api_token', 'registration_token', 'runner_url', 'runners']
    if not all(key in data for key in keys):
        return False

    # 检查URL和Token的类型
    if not all(isinstance(data[key], str) for key in
               ['gitlab_url', 'gitlab_api_token', 'registration_token', 'runner_url']):
        return False

    # 检查runners数组
    if not isinstance(data['runners'], list):
        return False

    # 检查每个runner的字段和类型
    runner_keys = ['description', 'tags', 'service', 'work_dir']
    for runner in data['runners']:
        if not all(key in runner for key in runner_keys):
            return False
        if not (isinstance(runner['description'], str) and isinstance(runner['tags'], list) and isinstance(
                runner['service'], str) and isinstance(runner['work_dir'], str)):
            return False
        if not all(isinstance(tag, str) for tag in runner['tags']):
            return False

    return True


def load_and_validate_json(file_path) -> Union[bool, dict]:
    try:
        with open(file_path, 'r') as file:
            data = json.load(file)
            if not validate_json(data):
                return False
            return data
    except (json.JSONDecodeError, FileNotFoundError, Exception):
        return False



import json


def input_runner_info():
    """ 交互式输入单个runner的配置信息 """
    description = input("请输入runner的描述: ")
    tags = input("请输入runner的标签（使用逗号分隔）: ").split(',')
    service = input("请输入runner的服务名: ")
    work_dir = input("请输入runner的工作目录路径: ")
    install_options = input("请输入安装选项（--option1 value1,--option2 value2...）: ").split(',')
    start_options = input("请输入启动选项（--option1 value1,--option2 value2...）: ").split(',')
    register_options = input("请输入注册选项（--option1 value1,--option2 value2...）: ").split(',')

    return {
        'description': description.strip(),
        'tags': [tag.strip() for tag in tags],
        'service': service.strip(),
        'work_dir': work_dir.strip(),
        'install_options': [option.strip() for option in install_options],
        'start_options': [option.strip() for option in start_options],
        'register_options': [option.strip() for option in register_options]
    }


def generator():
    """ 主函数，生成JSON配置文件 """
    gitlab_url = input("请输入GitLab URL: ")
    gitlab_api_token = input("请输入GitLab API token: ")
    registration_token = input("请输入注册token: ")
    runner_url = input("请输入GitLab Runner的下载URL: ")

    runners = []
    more_runners = True
    while more_runners:
        runners.append(input_runner_info())
        more_runners = input("是否添加更多的runner? (y/n): ").lower() == 'y'

    config = {
        'gitlab_url': gitlab_url,
        'gitlab_api_token': gitlab_api_token,
        'registration_token': registration_token,
        'runner_url': runner_url,
        'runners': runners
    }

    file_path = input("请输入配置文件保存的路径及名称（例如 C:/path/to/config.json）: ") or "./config.json"
    with open(file_path, 'w') as f:
        json.dump(config, f, indent=4)
    print(f"配置文件已保存到 {file_path}")
    return input("是否立即执行？（y/n）").lower() == "y"




import tkinter as tk
from tkinter import scrolledtext
import subprocess
import threading


def run_command():
    """ 运行命令行命令，并将输出实时显示在 Text 控件中。"""
    # 示例命令，这里应替换为需要执行的实际命令
    cmd = ['ping', 'www.google.com', '-n', '5']

    # 启动子进程执行命令
    process = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True)

    # 实时读取输出
    for line in process.stdout:
        text_area.insert(tk.END, line)
        text_area.see(tk.END)  # 自动滚动到底部
        root.update_idletasks()  # 更新GUI

    # 等待命令执行结束
    process.wait()

    # 将结束信息或错误信息输出到文本区域
    if process.returncode == 0:
        text_area.insert(tk.END, "\n命令执行完成！\n")
    else:
        error = process.stderr.read()
        text_area.insert(tk.END, f"\n发生错误：\n{error}")
    text_area.see(tk.END)
    root.update_idletasks()


def start_thread():
    """ 在新线程中运行命令，避免阻塞 GUI。"""
    threading.Thread(target=run_command).start()


# 创建主窗口
root = tk.Tk()
root.title("安装界面")

# 创建滚动文本区域用于显示输出，设置背景为黑色，前景（文字）为白色
text_area = scrolledtext.ScrolledText(root, width=80, height=20, bg='black', fg='white', insertbackground='white')
text_area.grid(column=0, row=0, padx=10, pady=10)

# 创建启动命令的按钮
start_button = tk.Button(root, text="开始安装", command=start_thread)
start_button.grid(column=0, row=1, padx=10, pady=10)

# 启动主事件循环
root.mainloop()



import ctypes
import sys

import tkinter as tk


def show_message(title, message):
    ctypes.windll.user32.MessageBoxW(0, message, title, 0x1000 | 0x40)


def is_admin():
    try:
        return ctypes.windll.shell32.IsUserAnAdmin()
    except:
        return False


def run_as_admin(argv=None):
    if argv is None and sys.argv:
        argv = sys.argv
    if hasattr(sys, '_MEIPASS'):
        arguments = map(str, argv[1:])
    else:
        arguments = map(str, argv)
    cmd = ' '.join(arguments)
    # ctypes.windll.shell32.ShellExecuteW(None, "runas", sys.executable, cmd, None, 0)
    show_message("错误", "请以管理员身份运行.")
    sys.exit()


def insert_tk_text(text_area, text):
    if not text_area:
        return
    text_area.insert(tk.END, text)
    text_area.see(tk.END)  # 确保滚动到最新插入的文本处



import tkinter as tk

import gitlab

from baosight.gitlab_tools.config import load_and_validate_json
from baosight.gitlab_tools.core import download_gitlab_runner, register_runner, \
    setup_runner_on_windows
from baosight.gui.gl_runner_installer import create_installation_ui
from baosight.tools.utils import show_message, is_admin, run_as_admin, insert_tk_text


def deploy(config_path=None, gui_objs=None):
    progress_bar = None
    root = None
    start_button = None
    text_area = None
    if gui_objs:
        progress_bar = gui_objs['progress_bar']
        config_button = gui_objs["config_button"]
        root = gui_objs['root']
        start_button = gui_objs['start_button']
        text_area = gui_objs['text_area']
        start_button.config(state=tk.DISABLED)
        config_path = config_button.invoke()
        if not config_path:
            start_button.config(state=tk.NORMAL)
            return
    config = load_and_validate_json(config_path)
    if not config:
        show_message("错误", "安装文件无效.")
        if gui_objs:
            start_button.config(state=tk.NORMAL)
        return
    download_gitlab_runner(config['runner_url'], text_area)

    gl = gitlab.Gitlab(config['gitlab_url'], private_token=config['gitlab_api_token'])
    for runner_info in config['runners']:
        runner_token = register_runner(
            gl,
            runner_info,
            config['registration_token'],
            text_area
        )
        if not runner_token:
            show_message("错误", "无法注册 runner.")
            return
        setup_runner_on_windows(
            runner_token,
            runner_info,
            config['gitlab_url'],
            text_area
        )
        print(f"Succeed Install: {runner_info['description']}\n")
        if gui_objs:
            progress_bar['value'] += 100 / len(config['runners'])
            insert_tk_text(text_area, f"Succeed Install: {runner_info['description']}\n")
            progress_bar.update()
    if gui_objs:
        start_button.config(text="完成", command=root.destroy)
        start_button.config(state=tk.NORMAL)


def main(config_path=None):
    if not is_admin():
        run_as_admin()
    print("Script is running with administrator privileges.")
    if config_path:
        deploy(config_path)
    else:
        create_installation_ui(deploy)



from argparse import ArgumentParser
from baosight.gitlab_tools.gitlab_runner import main
from baosight.gitlab_tools.gitlab_runner_config_generator import generator

class CLICommand:
    """安装gitlab runner
    安装gitlab runner,配置文件格式如下：
    {
      "gitlab_url": "http://127.0.0.1",
      "gitlab_api_token": "xxxxxxx",
      "registration_token": "xxxxxxx",
      "runner_url": "https://s3.dualstack.us-east-1.amazonaws.com/gitlab-runner-downloads/latest/binaries/gitlab-runner-windows-amd64.exe",
      "runners": [
        {
          "description": "Windows Runner",
          "tags": [
            "windows"
          ],
          "service": "gitlab-runner",
          "work_dir": "C:\\Runners\\Runner3"
        },
      ],
    }
    """

    @staticmethod
    def add_arguments(parser: ArgumentParser):
        add = parser.add_argument
        add("--quite", action='store_true', help="静默安装当前目录下的config.json")
        add("--create-config", dest="config", action='store_true', help="交互式生成配置文件")

    @staticmethod
    def run(args, parser):
        if args.config:
            if generator():
                main("config.json")
            return
        if args.quite:
            main("config.json")
        else:
            main()
